@@ -1,14 +1,12 @@
-"""
-URLs.
-"""
-from django.conf.urls import patterns, url
+from django.urls import re_path
 
-import views
+from . import views
 
-urlpatterns = patterns('admin_steroids.views',
-    url(
+urlpatterns = [
+    re_path(
         r'^(?P<app_name>[^/]+)/(?P<model_name>[^/]+)/field/(?P<field_name>[^/]+)/search/?',
         views.ModelFieldSearchView.as_view(),
         # If you override this, be sure to use the same name, so reverse() still works.
-        name='model_field_search'),
-)
+        name='model_field_search'
+    ),
+]
